import numpy as np

# -- Helper function for removing triggers outside a segment list
def find_in_segs(triglist, segs):
    indx = np.zeros(len(triglist), dtype=bool)
    for n in range(len(triglist)):
        gps = triglist[n]
        indx[n] = np.any( np.logical_and( gps>(segs['start']+8), gps<(segs['stop']-4)) )
    return indx


namestring = """
 GravEn_SimID                        SimHrss      SimEgwR2    GravEn_Ampl  Internal_x   Internal_phi   External_x   External_phi  External_psi  FrameGPS     EarthCtrGPS     SimName    
           SimHpHp      SimHcHc      SimHpHc    GEO      GEOctrGPS       GEOfPlus       GEOfCross  H1       H1ctrGPS        H1fPlus        H1fCross  H2       H2ctrGPS        H2fPlus     
   H2fCross  L1       L1ctrGPS        L1fPlus        L1fCross  V1       V1ctrGPS        V1fPlus        V1fCross 
"""

namelist = namestring.split()

data = np.recfromtxt('BurstMDC-BRST_S6-Log.txt', names=namelist)

waveform = data['SimName']
hrss = data['SimHrss']
phi = data['External_phi']
x = data['External_x']
theta = np.arccos(x)
dec = np.pi/2 - theta
gps = data['EarthCtrGPS']

sgname = 'SG153Q8d9'
wnbname = 'WNB_100_100_0d1'

loudsg =  [gps[i] for i in range(len(gps)) if (waveform[i] == sgname) and (gps[i] > 961545615) and (gps[i] < 971654415) ]

wnb = [gps[i] for i in range(len(gps)) if (waveform[i] == wnbname) and (gps[i] > 961545615) 
       and (gps[i] < 971654415) and (hrss[i]<3e-21)]

loudwnb = [gps[i] for i in range(len(gps)) if (waveform[i] == wnbname) and (gps[i] > 961545615) 
           and (gps[i] < 971654415) and (hrss[i]>3e-21)]

# ---------------------
# Read in segment lists
# ---------------------
segfmt = ['start', 'stop', 'dur']
h1segs = np.recfromtxt('H1_BURST_CAT2_LONG.txt', names=segfmt)
l1segs = np.recfromtxt('L1_BURST_CAT2_LONG.txt', names=segfmt)
v1segs = np.recfromtxt('V1_VSR3.txt', names=segfmt)

# -- Find which triggers are in the segment list somewhere
h1sg = find_in_segs(loudsg, h1segs)
l1sg = find_in_segs(loudsg, l1segs)
v1sg = find_in_segs(loudsg, v1segs)
ligoindx = np.logical_and(h1sg, l1sg)
indx = np.logical_and(ligoindx, v1sg)
goodsg = np.array(loudsg)[indx]
print "Found {0} SGs in good times".format(goodsg.size)

# -- Find which triggers are in the segment list somewhere
h1wnb = find_in_segs(wnb, h1segs)
l1wnb = find_in_segs(wnb, l1segs)
v1wnb = find_in_segs(wnb, v1segs)
ligoindx = np.logical_and(h1wnb, l1wnb)
indx = np.logical_and(ligoindx, v1wnb)
goodwnb = np.array(wnb)[indx]
print "Found {0} quiet WNB in good times".format(goodwnb.size)

# -- Find which triggers are in the segment list somewhere
h1 = find_in_segs(loudwnb, h1segs)
l1 = find_in_segs(loudwnb, l1segs)
v1 = find_in_segs(loudwnb, v1segs)
ligoindx = np.logical_and(h1, l1)
indx = np.logical_and(ligoindx, v1)
goodloudwnb = np.array(loudwnb)[indx]
print "Found {0} loud WNB in good times".format(goodloudwnb.size)

        
# -- Set how many triggers we want
N = 20
start = 21
stop = start + N

outfile = open('trigs_sg153.txt','w')
for time in goodsg[start:stop]:
    outfile.write("{0} \n".format(time) )
outfile.close()

outfile = open('trigs_qwnb.txt','w')
for time in goodwnb[start:stop]:
    outfile.write("{0} \n".format(time) )
outfile.close()

outfile = open('trigs_lwnb.txt','w')
for time in goodloudwnb[start:stop]:
    outfile.write("{0} \n".format(time) )
outfile.close()



# outfile = open('setup_sg153','w')
# for time in goodsg[0:N]:
#     outfile.write("./condor_setup.sh {0} \n".format(time) )
# outfile.close()

#outfile = open('setup_qwnb.sh','w')
#for time in goodwnb[0:N]:
#    outfile.write("./condor_setup.sh {0} \n".format(time) )
#outfile.close()

#outfile = open('setup_lwnb.sh','w')
#for time in goodloudwnb[0:N]:
#    outfile.write("./condor_setup.sh {0} \n".format(time) )
#outfile.close()
