import numpy as np

# -- Helper function for removing triggers outside a trigger list
def find_in_segs(triglist, segs):
    indx = np.zeros(len(triglist), dtype=bool)
    for n in range(len(triglist)):
        gps = triglist[n]
        indx[n] = np.any( np.logical_and( gps>(segs['start']+8), gps<(segs['stop']-4)) )
    return indx

namestring = """
  GravEn_SimID                        SimHrss      SimEgwR2    GravEn_Ampl  Internal_x   Internal_phi   External_x   External_phi  External_psi  FrameGPS     EarthCtrGPS     SimName               SimHpHp      SimHcHc      SimHpHc    GEO     GEOctrGPS       GEOfPlus       GEOfCross  H1       H1ctrGPS        H1fPlus        H1fCross  H2       H2ctrGPS        H2fPlus        H2fCross  L1       L1ctrGPS        L1fPlus        L1fCross  V1       V1ctrGPS        V1fPlus        V1fCross
"""

namelist = namestring.split()
data = np.recfromtxt('BurstMDC-ELPTC_S6-Log.txt', names=namelist)

waveform = data['SimName']
hrss = data['SimHrss']
phi = data['External_phi']
x = data['External_x']
theta = np.arccos(x)
dec = np.pi/2 - theta
gps = data['EarthCtrGPS']

sgname = 'SG153Q8d9'
wnbname = 'WNB_100_100_0d1'

loudsg =  [gps[i] for i in range(len(gps)) if (waveform[i] == sgname) and (gps[i] > 961545615) and (gps[i] < 971654415) ]

# ---------------------
# Read in segment lists
# ---------------------
segfmt = ['start', 'stop', 'dur']
h1segs = np.recfromtxt('H1_BURST_CAT2_LONG.txt', names=segfmt)
l1segs = np.recfromtxt('L1_BURST_CAT2_LONG.txt', names=segfmt)

# -- Find which triggers are in the segment list somewhere
h1sg = find_in_segs(loudsg, h1segs)
l1sg = find_in_segs(loudsg, l1segs)
indx = np.logical_and(h1sg, l1sg)
goodsg = np.array(loudsg)[indx]
print "Found {0} SGs in good times".format(goodsg.size)
        
# -- Set how many triggers we want
N = 40

outfile = open('trigs_elpsg153.txt','w')
for time in goodsg[0:N]:
    outfile.write("{0} \n".format(time) )
outfile.close()
