# -----------------------------------
# Example Condor setup script for BWB
# 2014, Jonah Kanner
# ----------------------------------
import os
import numpy as np
import subprocess

# ------------------
# -- Set paramters
# ------------------
topdir = '/home/jkanner/baysewave/PEC/mdc_v3'
# topdir = '/home/jkanner/baysewave/test'
mdcdir = '/home/jkanner/baysewave/svn/branches/condor_compile/burstinj/s6'
waveformList = ['elpsg153']
scaleList = ['0.15' , '0.3', '0.6', '1.2']
mdccache = '/home/jkanner/baysewave/svn/trunk/burstinj/s6/elptc_s6.cache'
mdclog = '/home/jkanner/baysewave/svn/trunk/burstinj/s6/BurstMDC-ELPTC_S6-Log.txt'
bwb = '/home/jkanner/baysewave/svn/branches/condor_compile/src/BayesWaveBurst'
ifoList = ['H1', 'L1', 'V1']
frtypeList = ['H1_LDAS_C02_L2', 'L1_LDAS_C02_L2', 'HrecV2']

bwbargsfmt = """--ifo H1 --H1-flow 40 --H1-channel H1:LDAS-STRAIN \
       --ifo L1 --L1-flow 40 --L1-channel L1:LDAS-STRAIN \
       --ifo V1 --L1-flow 40 --V1-channel V1:h_16384Hz \
       --H1-cache {wavedir}/H1.cache \
       --L1-cache {wavedir}/L1.cache \
       --V1-cache {wavedir}/V1.cache \
       --trigtime {gps} --srate 1024 --seglen 4 \
       --bayesLine --PSDstart {gps} --PSDlength 4 \
       --Niter 2000000 --NCmin 15  \
       --bayesLine --gnuplot \
       --MDC-channel [H1:GW-H,L1:GW-H,V1:GW-16K]   \
       --MDC-cache [{cachefile},{cachefile},{cachefile}] \
       --MDC-prefactor {scale} \
"""

# -- Alternate parameters
# waveformList = ['sg153', 'lwnb', 'qwnb']
# mdccache = '/home/jkanner/baysewave/svn/trunk/burstinj/s6/brst_s6.cache'
# scaleList = ['0.15', '0.3', '0.6', '1.2']
# topdir = '/home/jkanner/baysewave/test'

# ---------------------------------
# -- Define templates for run files
# ---------------------------------
submit_str = """
executable={bwb}
universe=standard
arguments={bwbargs}
output={top}/condorOut.txt
error={top}/condorError.txt
log={top}/condorLog.txt
notification=never
should_transfer_files=YES
when_to_transfer_output = ON_EXIT
stream_error=True
stream_output=True
queue 1
"""

# -----------------
# Begin Main Script
# -----------------
dagfile = open( os.path.join(topdir, 'submit_all.sh'), 'w')

for waveform in waveformList:
    trigfile = os.path.join(mdcdir, "trigs_{0}.txt".format(waveform))
    trigList = np.loadtxt(trigfile)
    wavedir = os.path.join(topdir, waveform)
    if not os.path.exists(wavedir): os.makedirs(wavedir)

    # ------------------------------------------------
    # Call LIGO Data find for whole waveform directory
    # ------------------------------------------------
    start = int(trigList.min()) - 50
    end   = int(trigList.max()) + 50
    for ifo, frtype in zip(ifoList,frtypeList):
        cachefilefmt = os.path.join(wavedir, '{0}.cache')
        ldfcmd = "ligo_data_find --observatory {o} --type {frtype} -s {start} -e {end} --lal-cache | grep file > {cachefile}".format(o=ifo[0], frtype=frtype, cachefile = cachefilefmt.format(ifo), start=start, end=end)
        print "Calling LIGO data find ..."
        print ldfcmd
        subprocess.call(ldfcmd, shell=True)

    for scale in scaleList:
        scaledir = os.path.join(wavedir, scale)
        if not os.path.exists(scaledir): os.makedirs(scaledir)
        
        for trig in trigList:
            trig = str(trig)
            trigdir = os.path.join(scaledir, 'job_'+trig)
            if not os.path.exists(trigdir): os.makedirs(trigdir)
            
            bwbargs = bwbargsfmt.format(gps=trig, wavedir=wavedir, cachefile=mdccache, scale=scale) 
            submitname = os.path.join(trigdir, 'submit.txt')
            submitfile = open( submitname, 'w' )
            submitfile.write(submit_str.format(top=trigdir, bwb=bwb, bwbargs=bwbargs))
            submitfile.close()
            
            dagfile.write("cd {0}\n".format(trigdir))
            dagfile.write("condor_submit submit.txt\n\n")

dagfile.close()
print "Thank you, come again"
