import numpy as np
import matplotlib.pyplot as plt
from scipy import stats
from time import time

plt.ion()
plt.close('all')

livetime = 22*3e7
names = ['bn', 'bg', 'bs', 'zn', 'zg', 'zs', 'rho']
data = np.recfromtxt('rho100_evidence_stacked.dat', names=names)
lnsg = data['bs'] - data['bg']
bayes_sg = np.exp(lnsg)

bins = np.arange(0,10,0.01)
bins = 10**bins
count,bins,patches = plt.hist(bayes_sg, bins=bins, cumulative=-1, log=True)
plt.figure()

bintoo = [np.mean([bins[n], bins[n+1]]) for n in range(bins.size-1)]
plt.semilogy(np.log10(bintoo), count / livetime)
plt.ylabel('Cumulative rate of glitches (Hz)')
plt.xlabel('log10($B_{sg}$)')


# ----- Model Background
N_gl = 22.0 * 3e7 * 0.001
B_inf = N_gl
model = np.log(B_inf) - np.log(bintoo)
plt.semilogy(np.log10(bintoo), model/livetime, 'r')
print "The total number of glitches is {0}".format(N_gl)

# exp_gl = N_gl * np.exp(-np.array(bintoo))
# plt.semilogy(bintoo, exp_gl/livetime)

# -- Upper limit on background
plt.axvline(np.log10(N_gl), color='g')
# plt.axvline(13, color='k')
plt.ylim(0.1/livetime, 100/livetime)
plt.xlim(0,7)
plt.grid()


# -------------------------
# Begin simulating glitches
# -------------------------
globstep = 0.2
xstar = 5.0*np.sqrt(3)
globx = np.arange(0,100, globstep)
pdf = globx/xstar**2*np.exp(-globx/xstar)
globcdf = pdf.cumsum()*globstep

class glitch_amp(stats.rv_continuous):

    def __init__(self):
        stats.rv_continuous.__init__(self, a=0, b=99)
    
    def _cdf(self, xr):        
        indx = np.where( np.logical_and(globx>=xr, globx<xr+globstep) )
        return globcdf[indx]
        
    #def plotme(self):
    #    plt.figure()
    #    plt.plot(self.x, self.pdf)


def get_occum(a,f):
    return 7240*(a**2)/f

def freq_rvs(size):
    onescale = np.random.rand(size)
    shift = 50
    scale = 512-shift
    return onescale*scale + shift

def flat_amp(size, maxamp=50):
    onescale = np.random.rand(size)
    return onescale*maxamp

# simnum = N_gl

# -- Create a population of glitches
factor = 1.0
simnum=N_gl/factor
simtime = livetime/factor

gl = glitch_amp()
t0 = time()
# amplitude = gl.rvs(size=simnum)
t1 = time()
print "Took {0} seconds".format(t1-t0)
freq = freq_rvs(simnum)
amplitude = flat_amp(simnum, 50)

# -- Pop 2
amp2 = flat_amp(simnum, 1000)

occumList = get_occum(amplitude, freq)
occum2    = get_occum(amp2, freq)

# -- Occum 3 has half the population w/ 2 wavelets
occum3    = np.zeros(occumList.size)
occum3[::2] = occumList[::2]**2 
occum3[1::2] = occumList[1::2]

# -- Randomly remove glitches based on occum factor
randvec = np.random.rand(occumList.size)
passList = np.where(randvec < (1.0/occumList) )

# -- Remove pop 2
randvec = np.random.rand(occumList.size)
passtwo = np.where(randvec < (1.0/occum2) )

# -- Remove pop 3
randvec = np.random.rand(occumList.size)
passthree = np.where(randvec < (1.0/occum3) )


# -- Hisogram the simulated glitches
count_sim,bins,patches = plt.hist(occumList[passList], bins=bins, cumulative=-1, log=True)

# -- Hisogram the simulated glitches
sim2,bins,patches = plt.hist(occum2[passtwo], bins=bins, cumulative=-1)
sim3,bins,patches = plt.hist(occum3[passthree], bins=bins, cumulative=-1)

bintoo = np.array(bintoo)
lnbin = np.log(bintoo)

plt.figure()
plt.semilogy(lnbin, count_sim / simtime, 'g', label='SNR 50')
plt.semilogy(lnbin, sim2 / simtime, 'r', label='SNR 1000')
plt.semilogy(lnbin, count / livetime, 'b', label='S6D')
plt.semilogy(lnbin, sim3 / simtime, 'c', label='2 wavelets')

plt.semilogy(lnbin, N_gl / (livetime*bintoo), color='k', label='Limit')
plt.axvline(np.log(N_gl), color='k', label='Scaling Law')
plt.ylabel('Cumulative rate of glitches (Hz)')
plt.xlabel('LN($B_{sg}$)')

plt.xlim(2,15)
plt.ylim(1e-9, 1e-5)
plt.grid()
plt.legend()



