import matplotlib
matplotlib.use('Agg')
import numpy as np
import matplotlib.pyplot as plt
import glob
import os
import sys
from pylal.xlal.datatypes.ligotimegps import LIGOTimeGPS
from pylal.xlal.date import XLALGreenwichSiderealTime
import find_trig as ft
import healpy as hp
from subprocess import call
# from scipy.optimize import curve_fit
import acor
from pylal import bayespputils
import sky_area.sky_area_clustering as sky
# import skyview
from scipy import stats


# -- Close any open figures
plt.close('all')


# -- Set up lists of results
glitch = []
signal = []
glitcherr = []
scale = []
snrList = []
goodJobList = []
norm = []
skydict = {}

# -- Get directory list
topdir = os.getcwd()
dirList = glob.glob('*/job*')
# dirList = [os.path.abspath(directory) for directory in dirList]

# -- Set normalization
norm = 2.5e-21
if topdir.find('lwnb')>0:
    norm = norm*np.sqrt(2)

# -- Loop over jobs
for job in dirList:
    os.chdir(job)
    print "Entered", os.getcwd()
    jobname = glob.glob('*bayeswave.run')[0]
    bayeswave = open(jobname, 'r')
    # ---------------------------------------
    # Extract information from bayeswave.run
    # ---------------------------------------
    ifoNames = []
    for line in bayeswave:
        spl = line.split()
        if len(spl) < 1: continue
        
        # Determine names of IFOs and MDC scale factor
        if spl[0] == 'Command' and spl[1] == 'line:':
            for index, splElement in enumerate(spl):
                if splElement == '--ifo':
                    ifoNames.append(spl[index+1]) 
                if splElement == '--MDC-prefactor':
                    print spl
                    scale.append(float(spl[index+1]))
                    print "At this point, scale is {0}".format(scale)
            continue         

        del spl[-1]
        # Determine number of IFOs
        if ' '.join(spl) == 'number of detectors':
            spl = line.split()
            ifoNum = spl[3]
            continue         
        # Determine gps trigger time
        if ' '.join(spl) == 'trigger time (s)':
            spl = line.split()
            gps = spl[3]
            break
    ifoNum = int(ifoNum)
    if ifoNum == 1:
        ifoList = ['0']
    elif ifoNum == 2:
        ifoList = ['0', '1']
    elif ifoNum == 3:
        ifoList = ['0', '1', '2']
    
    # ----------------------------------
    # Determine name of job 
    # ----------------------------------
    jobname = jobname.split('_')
    del jobname[-1]
    if not len(jobname) == 0:
        jobname = '_'.join(jobname)+'_'
    else:
        jobname = ''

    # -------------------
    # Read evidence file
    # -------------------
    try:
        infile = open(str(jobname)+'evidence.dat', 'r')
    except:
        os.chdir(topdir)
        continue
    
    for line in infile:
        spl = line.split()
        if spl[0] == 'noise':
            sig_noise = -float(spl[1])
            signal.append(sig_noise)
        if spl[0] == 'glitch':
            sig_gl = -float(spl[1])
            error_a = float(spl[2])
            glitch.append(sig_gl)
        if spl[0] == 'signal':
            error_b = float(spl[2])
    infile.close()

    glitcherr.append(np.sqrt(error_a + error_b))
    

    # -- Parse log file
    infile = open('condorOut.txt', 'r')
    redo = False
    snr = []
    print "Parsing log file..."
    for line in infile:
        if line.find('Recompute') > 0:
            print "Found recompute line"
            print line
            redo = True
        if redo:
            if line.find('Injected SNR') > -1:
                print "Found line with SNR"
                print line
                snr.append(float(line.split()[2]))
    if len(snr) == 0:
        sys.exit("No injections found in this run!")
    snrList.append(np.sqrt(snr[0]**2 + snr[1]**2 ))
    goodJobList.append(job)
    infile.close()

    # -- Return to top level directory, continue through list of jobs
    plt.close('all')
    os.chdir(topdir)

# -- Need to set the matplotlib figure size
plt.rcParams['figure.figsize'] = 8, 6



# ----------------------
# Make histogram of SNRs
# ----------------------
plt.figure()
plt.hist(snrList)
plt.xlabel('SNR')
plt.ylabel('Number')
plt.savefig('snr_hist.png')
plt.close()


# --------------------
# Make SNR table 
# -------------------
snrTable = open('snrTable_fdf.html', 'w')
snrTable.write('<html><body>')
snrTable.write('<h1>Bayes Wave Burst Event Overviews</h1>')
snrTable.write('<table>')
snrTable.write('<tr><b><td>SNR <td>Job    <td>Evidence(S/N)  <td>Evidence(S/G)</b></tr>')

# -- Make tuple, sort by SNR
raw_eventList = zip(snrList, signal, glitch, goodJobList)
eventList =  sorted(raw_eventList, key=lambda event: event[0], reverse=True)

for (snr, sn, sg, job) in eventList:
    snrTable.write("""<tr>
<td>{0:.2f}  
<td><a href='{1}'>{1}</a>  
<td>{2:.2f}  
<td>{3:.2f}
</tr>
""".format(snr, job, sn, sg) )
snrTable.write('</table></html></body>')
snrTable.close()



# -----------------------------
# -- Try making efficiency plot
# ----------------------------
glitch = np.array(glitch)
signal = np.array(signal)
scale = np.array(scale)
goodJobList = np.array(goodJobList)
glitcherr = np.array(glitcherr)
uniq_scale = np.array(list(set(scale)))
snrList = np.array(snrList)
eff = np.array([])


for s in uniq_scale:
    indx = np.logical_and(scale == s, snrList<40)
    this_signal = signal[indx]
    this_glitch = glitch[indx]
    this_glitcherr = glitcherr[indx]
    total_inj = this_signal.size
    detected = np.logical_and(this_signal>0, (this_glitch+3*this_glitcherr)>0).sum()
    eff = np.append(eff, detected*1.0/total_inj)

plt.semilogx(uniq_scale*norm, eff, 'o')
plt.ylim((0,1))
plt.xlim(( norm*uniq_scale.min()*0.9, norm*uniq_scale.max()*1.1 ))
plt.xlabel('Scale factor')
plt.ylabel('Detection Efficiency')
plt.grid(which='both')
plt.savefig('fdf_eff.png')
plt.close()

# -------------------------------------
# Try making a list of false dismissals
# -------------------------------------
# -- A false dismissal means snr > 15 and glitch < 0
# -- That is, we're looking for loud signals classified as glitches
indx = np.where(np.logical_and( snrList>15, (glitch+(3*glitcherr))<0 ))[0]

snrTable = open('fdfTable.html', 'w')
snrTable.write('<html><body>')
snrTable.write('<h1>Bayes Wave Burst Events - False dismissals</h1>')
snrTable.write('<table>')
snrTable.write('<tr><b><td>SNR <td>Job    <td>Evidence(S/N)  <td>Evidence(S/G)</b></tr>')

# -- Make tuple, sort by SNR
raw_eventList = zip(snrList[indx], signal[indx], glitch[indx], goodJobList[indx], glitcherr[indx])
eventList =  sorted(raw_eventList, key=lambda event: event[2], reverse=False)

for (snr, sn, sg, job, err) in eventList:
    snrTable.write("""<tr>
<td>{0:.2f}  
<td><a href='{1}/pec.html'>{1}</a>  
<td>{2:.2f}  
<td>{3:.2f}
<td>+/-{4:.2f}
</tr>
""".format(snr, job, sn, sg, err) )
snrTable.write('</table></html></body>')
snrTable.close()



# -----------------------
# Make top level web page
# ----------------------
outfile = open('fdf.html', 'w')
outfile.write('<html><body>')
outfile.write('<h1>BayesWave Burst Injection Run</h1>')
outfile.write("<h4><a href='./snrTable_fdf.html'>Link to the SNR table for an overview of each event.</a></h4><br/>")
outfile.write("<h4><a href='./skyTable.html'>Link to the SKY table for position reconstructions.</a></h4><br/>")
outfile.write("<h4><a href='./fdfTable.html'>Link to FALSE DISMISSALS list</a></h4><br/>")
outfile.write("<h3>Efficiency Curve</h3>")
outfile.write("<img src='fdf_eff.png' alt='eff.png' width=600>")

outfile.write("<img src='./snr_hist.png' alt='snr_hist' width=600>")
outfile.close()
 
