import matplotlib as mat
mat.use('Agg')
import matplotlib.pyplot as plt
import numpy as np
from glue.ligolw import ligolw
from glue.ligolw import ligolw
from glue.ligolw import table
from glue.ligolw import lsctables
from glue.ligolw import utils as ligolw_utils


# -----------------------------
# Read XML injection parameters
# ------------------------------
filename = 'IMBHB-MDC-150-150_10Hz_taper_start.xml'
lsctables.use_in(ligolw.LIGOLWContentHandler)
xmldoc = ligolw_utils.load_filename(filename, contenthandler = ligolw.LIGOLWContentHandler, verbose = True)
tbl = lsctables.SimInspiralTable.get_table(xmldoc)

# -------------------
# Extract GPS and number of each injection
# -------------------
gps = np.zeros(len(tbl))
number = np.zeros(len(tbl), dtype=int)
for indx, inj in enumerate(tbl):
    gps[indx] = inj.h_end_time + 1e-9*inj.h_end_time_ns
    print gps[indx]
    number_str = str(inj.simulation_id)
    number[indx] = number_str.split(':')[2]
    print number[indx]

# ---------------
# Read list of recovered injections
# ---------------
recovered = np.recfromtxt('recovered_signals_150.txt', names=True)
rec_timeList = recovered['GPSL1']
rec_rhoList  = recovered['rho']
rec_snrList  = recovered['iSNR']

rec_num = np.zeros(rec_timeList.shape)
for q, rec_time in enumerate(rec_timeList):

    indx = np.argmin(np.abs(gps-rec_time))
    rec_num[q] = number[indx]


# ----------------------
# Make list of BG triggers
# -----------------------
trigfile = open('/home/jkanner/sergey/bg.txt', 'r')
Nbg = 1000
bg_rho = np.ones(Nbg, dtype=float)
count = 0
for line in trigfile:
    spl = line.split()
    if spl[0] == '#': continue
    bg_rho[count] = float(spl[2])
    count += 1    
    if count >= Nbg: break
trigfile.close()

livetime = 1896.5*3e7  # See cWB background page

three_sig = (1 - 0.9973)*(1.0/3e7)
four_sig  = (1-0.999936)*(1.0/3e7) 
five_sig  = (1-0.99999942)*(1.0/3e7)

# -- Get cumulative histogram of rho in background
number, bins, patches = plt.hist(bg_rho, bins=np.arange(6,110,0.1), cumulative=-1)
bins = bins[0:-1]
plt.close('all')
bg_rate = number*1.0/livetime

# -- Get fraction of deteted events for each rho threshold
num_detected = np.zeros(bins.shape)
for indx, thresh in enumerate(bins):
    num_detected[indx] = (rec_rhoList > thresh).sum()

short_num_det = np.zeros(bins.shape)
for indx, thresh in enumerate(bins):
    short_num_det[indx] = (rec_rhoList[0:500] > thresh).sum()

# -- Plot detection fraction as a function of FAR
plt.figure()
plt.semilogx(bg_rate, num_detected*1.0/rec_rhoList.size, label='CWB')
plt.semilogx(bg_rate, short_num_det*1.0/500, 'k', label='CWB 500')
plt.axvline(three_sig, c='g', label='3 sig')
plt.axvline(four_sig, c='y', label='4 sig')
plt.axvline(five_sig, c='r', label='5 sig')
plt.legend()
plt.grid()
plt.xlabel('FAR (Hz)')
plt.ylabel('Fraction of events recovered')
plt.savefig('cwb_roc.png')


# ---------------
# Make output file
# ----------------

# np.savetxt( 'triglist_150.txt', zip(rec_timeList, rec_num))

 
