#!/usr/bin/python

import numpy as np
import time
import sys, json
from ligo.gracedb.rest import GraceDb
import os
import subprocess

# ----------------
# Set Parameters
# ----------------
triglist = 'jonah_lag.txt'
topdir = '/home/jkanner/baysewave/PEC/sergey/bg_v2'
cachedir = os.path.join(topdir, 'cachedir')
bwb = '/home/jkanner/baysewave/svn/trunk/src/BayesWaveBurst'
ifoList = ['H1', 'L1']
frtypeList  = ['T1200307_V4_EARLY_RECOLORED_V2', 'T1200307_V4_EARLY_RECOLORED_V2']

# ------------------------------------------
# Read list of GPS times and trigger numbers
# ------------------------------------------
names = ['gps', 'L1_lag']
data = np.recfromtxt(triglist, names=names)
gpsList = data['gps']
lagList = data['L1_lag']

# -----------------
# BWB arguments 
# -----------------
bwbargsfmt = """--ifo H1 --H1-flow 16 --H1-channel H1:LDAS-STRAIN  \
       --ifo L1 --L1-flow 16 --L1-channel L1:LDAS-STRAIN  \
       --H1-cache {cachedir}/s6recolored_H1.cache \
       --L1-cache {cachedir}/s6recolored_L1.cache \
       --trigtime {gps} --srate 1024 --seglen 4 \
       --bayesLine --PSDstart {gps} --PSDlength 4 \
       --outputDir {trigdir}  \
       --L1-timeslide {lag}
"""
#       --Niter 2000000 --NCmin 35 --NCmax 35  \
#       --MDC-channel [H1:GW-H,L1:GW-H]   \
#       --MDC-cache [{cachefile},{cachefile}] \
#        --MDC-prefactor {scale} \

# ----------
# Template for bwb submit file
# ----------
submit_str = """
executable={bwb}
universe=standard
arguments={bwbargs}
output={top}/condorOut.txt
error={top}/condorError.txt
log={top}/condorLog.txt
notification=never
should_transfer_files=YES
when_to_transfer_output = ON_EXIT
stream_error=True
stream_output=True
queue 1
"""

# ----------------------------
# Template for BWP submit file
# ----------------------------
post_submitstr = """
executable={top}/pp.sh
universe=vanilla
arguments=""
output={top}/ppOut.txt
error={top}/ppError.txt
log={top}/ppLog.txt
notification=never
should_transfer_files=YES
when_to_transfer_output = ON_EXIT
stream_error=True
stream_output=True
RequestMemory = 2000
queue 1
"""


# ----------------------------
# Template for SKYMAP submit file
# ----------------------------
sky_submitstr = """
executable={top}/sky.sh
universe=vanilla
arguments=""
output={top}/skyOut.txt
error={top}/skyError.txt
log={top}/skyLog.txt
notification=never
should_transfer_files=YES
when_to_transfer_output = ON_EXIT
stream_error=True
stream_output=True
RequestMemory = 2000
queue 1
"""


# ------------------
# Call LIGO Data find
# -------------------
start = 966383961 
end   = 971557939
if not os.path.exists(cachedir): os.makedirs(cachedir)

for ifo, frtype in zip(ifoList,frtypeList):
    cachefilefmt = os.path.join(cachedir, 's6recolored' + '_{0}.cache')
    ldfcmd = "ligo_data_find --observatory {o} --type {frtype} -s {start} -e {end} --lal-cache | grep file > {cachefile}".format(o=ifo[0], frtype=frtype, cachefile = cachefilefmt.format(ifo), start=start, end=end)
    print "Calling LIGO data find ..."
    print ldfcmd
    subprocess.call(ldfcmd, shell=True)

# -- Open DAG file
dagfile = open( os.path.join(topdir, 'dagfile.dag'), 'w')

# -------------------------
# Create Condor submit file
# ------------------------
for gps, lag in zip(gpsList, lagList)[0:5]:
    number = float(gps)
    trigdir = os.path.join(topdir, 'job_' + str(number) + '_' + str(lag) )
    if not os.path.exists(trigdir): os.makedirs(trigdir)

    # -- Create BWB submit file
    bwbargs = bwbargsfmt.format(gps=gps, cachedir=cachedir, trigdir=trigdir, lag=lag)
    submitname = os.path.join(trigdir, 'submitBWB.txt')
    submitfile = open(submitname, 'w')
    submitfile.write(submit_str.format(top=trigdir, bwb=bwb, bwbargs=bwbargs))
    submitfile.close()

    # -- Create BWP submit file
    post_submitname = os.path.join(trigdir, 'submitBWP.txt')
    submitfile = open(post_submitname, 'w')
    submitfile.write(post_submitstr.format(top=trigdir))
    submitfile.close()

    # -- Create BWP shell script
    svn = '/home/jkanner/baysewave/svn/trunk'
    ppname = os.path.join(trigdir, 'pp.sh')
    doitfile = open(ppname, 'w')
    doitfile.write("""#! /bin/sh
cd {directory}
source /home/jkanner/.bashrc.mine
python {svn}/postprocess/s6/run_bwp.py
python {svn}/postprocess/megaplot.py
""".format(directory=trigdir, svn=svn))
    doitfile.close()

    # -- Create SKY submit file
    sky_submitname = os.path.join(trigdir, 'submitsky.txt')
    submitfile = open(sky_submitname, 'w')
    submitfile.write(sky_submitstr.format(top=trigdir))
    submitfile.close()

    # -- Create SKY shell script
    ppname = os.path.join(trigdir, 'sky.sh')
    doitfile = open(ppname, 'w')
    doitfile.write("""#! /bin/sh
cd {directory}
source /home/jkanner/.bashrc.mine
python {svn}/postprocess/skymap/megasky.py 
""".format(directory=trigdir, svn=svn))
    doitfile.close()


    # -- Write the dag file

    graceid = 'job_' + str(number) + '_' + str(lag)

    dagfile.write("JOB {graceid} {submitname}\n".format(graceid=graceid, submitname=submitname))
    dagfile.write("RETRY {graceid} 1 \n\n".format(graceid=graceid))

    dagfile.write("JOB {graceid}_post {submitname}\n".format(graceid=graceid, submitname=post_submitname))
    dagfile.write("RETRY {graceid}_post 3 \n\n".format(graceid=graceid))

    dagfile.write("JOB {graceid}_sky {submitname}\n".format(graceid=graceid, submitname=sky_submitname))
    dagfile.write("RETRY {graceid}_sky 10 \n\n".format(graceid=graceid))

    dagfile.write("PARENT {graceid} CHILD {graceid}_post {graceid}_sky \n".format(graceid=graceid))

dagfile.close()

# -- Launch the DAG!
# os.chdir(trigdir)
# subprocess.call('condor_submit_dag dagfile.dag', shell=True)



